@@ -1,10 +1,10 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000 Free Software Foundation, Inc.
+    Copyright (C) 1999-2000, 2007, 2009 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,15 +13,24 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+/**
+ * \addtogroup PedException
+ * @{
+ */
+
+/** \file exception.h */
+
 #ifndef PED_EXCEPTION_H_INCLUDED
 #define PED_EXCEPTION_H_INCLUDED
 
 typedef struct _PedException PedException;
 
+/**
+ * Exception type
+ */
 enum _PedExceptionType {
 	PED_EXCEPTION_INFORMATION=1,
 	PED_EXCEPTION_WARNING=2,
@@ -32,6 +41,9 @@ enum _PedExceptionType {
 };
 typedef enum _PedExceptionType PedExceptionType;
 
+/**
+ * Option for resolving the exception
+ */
 enum _PedExceptionOption {
 	PED_EXCEPTION_UNHANDLED=0,
 	PED_EXCEPTION_FIX=1,
@@ -46,21 +58,25 @@ typedef enum _PedExceptionOption PedExceptionOption;
 #define PED_EXCEPTION_OK_CANCEL	    (PED_EXCEPTION_OK + PED_EXCEPTION_CANCEL)
 #define PED_EXCEPTION_YES_NO	    (PED_EXCEPTION_YES + PED_EXCEPTION_NO)
 #define PED_EXCEPTION_YES_NO_CANCEL (PED_EXCEPTION_YES_NO \
-						+ PED_EXCEPTION_CANCEL)
-#define PED_EXCEPTION_IGNORE_CANCEL (PED_EXCEPTION_IGNORE		\
+				     + PED_EXCEPTION_CANCEL)
+#define PED_EXCEPTION_IGNORE_CANCEL (PED_EXCEPTION_IGNORE \
 				     + PED_EXCEPTION_CANCEL)
 #define PED_EXCEPTION_RETRY_CANCEL  (PED_EXCEPTION_RETRY + PED_EXCEPTION_CANCEL)
-#define PED_EXCEPTION_RETRY_IGNORE_CANCEL (PED_EXCEPTION_RETRY		\
+#define PED_EXCEPTION_RETRY_IGNORE_CANCEL (PED_EXCEPTION_RETRY \
 					   + PED_EXCEPTION_IGNORE_CANCEL)
 #define PED_EXCEPTION_OPTION_FIRST PED_EXCEPTION_FIX
 #define PED_EXCEPTION_OPTION_LAST PED_EXCEPTION_CANCEL
 
+/**
+ * Structure with information about exception
+ */
 struct _PedException {
-	char*			message;
-	PedExceptionType	type;
-	PedExceptionOption	options;	/* or'ed list of options that
+	char*			message;	/**< text describing what the event was */
+	PedExceptionType	type;		/**< type of exception */
+	PedExceptionOption	options;	/**< ORed list of options that
 						   the exception handler can
-						   return */
+						   return (the ways an exception
+						   can be resolved) */
 };
 
 typedef PedExceptionOption (PedExceptionHandler) (PedException* ex);
@@ -71,6 +87,8 @@ extern char* ped_exception_get_type_string (PedExceptionType ex_type);
 extern char* ped_exception_get_option_string (PedExceptionOption ex_opt);
 
 extern void ped_exception_set_handler (PedExceptionHandler* handler);
+extern PedExceptionHandler *ped_exception_get_handler(void);
+
 extern PedExceptionOption ped_exception_default_handler (PedException* ex);
 
 extern PedExceptionOption	ped_exception_throw (PedExceptionType ex_type,
@@ -93,3 +111,6 @@ extern void			ped_exception_fetch_all ();
 extern void			ped_exception_leave_all ();
 
 #endif /* PED_EXCEPTION_H_INCLUDED */
+
+/** @} */
+
