@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998, 1999, 2000, 2002, 2007 Free Software Foundation, Inc.
+    Copyright (C) 1998-2000, 2002, 2007, 2009 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 */
 
 #include <config.h>
-#include "fat.h" 
+#include "fat.h"
 
 #ifndef DISCOVER_ONLY
 
@@ -62,7 +62,7 @@ PedSector
 fat_max_cluster_size (FatType fat_type) {
 	switch (fat_type) {
 		case FAT_TYPE_FAT12: return 1;	/* dunno... who cares? */
-		case FAT_TYPE_FAT16: return 32768/512;
+		case FAT_TYPE_FAT16: return 65536/512;
 		case FAT_TYPE_FAT32: return 65536/512;
 	}
 	return 0;
@@ -135,8 +135,8 @@ fat_check_resize_geometry (const PedFileSystem* fs,
 			  "partition to this size.  Currently, only %s is "
 			  "free."),
 			needed, have);
-		ped_free (needed);
-		ped_free (have);
+		free (needed);
+		free (have);
 		return 0;
 	}
 
@@ -328,7 +328,7 @@ fat_calc_align_sectors (const PedFileSystem* new_fs,
 	PedSector	align;
 
 	new_meta_data_size
-		= fat_min_reserved_sector_count (new_fs_info->fat_type) 
+		= fat_min_reserved_sector_count (new_fs_info->fat_type)
 		  + new_fs_info->fat_sectors * 2;
 
 	if (new_fs_info->fat_type == FAT_TYPE_FAT16)
