@@ -1,10 +1,10 @@
 /*
     libparted
-    Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+    Copyright (C) 1998-2001, 2007-2009 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,10 +13,10 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <config.h>
 #include <string.h>
 #include <uuid/uuid.h>
 
@@ -44,9 +44,9 @@ fat_alloc (const PedGeometry* geom)
 	return fs;
 
 error_free_type_specific:
-	ped_free (fs->type_specific);
+	free (fs->type_specific);
 error_free_fs:
-	ped_free (fs);
+	free (fs);
 error:
 	return NULL;
 }
@@ -69,7 +69,7 @@ fat_alloc_buffers (PedFileSystem* fs)
 	return 1;
 
 error_free_buffer:
-	ped_free (fs_info->buffer);
+	free (fs_info->buffer);
 error:
 	return 0;
 };
@@ -79,16 +79,16 @@ fat_free_buffers (PedFileSystem* fs)
 {
 	FatSpecific*	fs_info = FAT_SPECIFIC (fs);
 
-	ped_free (fs_info->cluster_info);
-	ped_free (fs_info->buffer);
+	free (fs_info->cluster_info);
+	free (fs_info->buffer);
 }
 
 void
 fat_free (PedFileSystem* fs)
 {
 	ped_geometry_destroy (fs->geom);
-	ped_free (fs->type_specific);
-	ped_free (fs);
+	free (fs->type_specific);
+	free (fs);
 }
 
 int
@@ -190,7 +190,6 @@ static int
 _init_fats (PedFileSystem* fs)
 {
 	FatSpecific*	fs_info = FAT_SPECIFIC (fs);
-	int		i;
 	FatCluster	table_size;
 
 	table_size = fs_info->fat_sectors * 512
@@ -235,7 +234,7 @@ fat_open (PedGeometry* geom)
 
 	if (!_init_fats (fs))
 		goto error_free_fs;
-	if (!fat_alloc_buffers (fs)) 
+	if (!fat_alloc_buffers (fs))
 		goto error_free_fat_table;
 	if (!fat_collect_cluster_info (fs))
 		goto error_free_buffers;
@@ -266,12 +265,15 @@ fat_root_dir_clear (PedFileSystem* fs)
  * with /dev/random) number.  Unfortunately, we can only use 4 bytes of it
  */
 static uint32_t
-_gen_new_serial_number ()
+_gen_new_serial_number (void)
 {
-	uuid_t		uuid;
+	union {
+		uuid_t uuid;
+		uint32_t i;
+	} uu32;
 
-	uuid_generate (uuid);
-	return * (uint32_t*) &uuid [0];
+	uuid_generate (uu32.uuid);
+	return uu32.i;
 }
 
 PedFileSystem*
@@ -360,7 +362,7 @@ fat_create (PedGeometry* geom, FatType fat_type, PedTimer* timer)
 	if (!fs_info->fat)
 		goto error_free_fs;
 	fat_table_set_cluster_count (fs_info->fat, fs_info->cluster_count);
-	if (!fat_alloc_buffers (fs)) 
+	if (!fat_alloc_buffers (fs))
 		goto error_free_fat_table;
 
 	if (fs_info->fat_type == FAT_TYPE_FAT32) {
@@ -537,7 +539,7 @@ fat_check (PedFileSystem* fs, PedTimer* timer)
 					!= PED_EXCEPTION_IGNORE)
 				goto error;
 		}
-	} 
+	}
 
 	if (fs_info->fat_type == FAT_TYPE_FAT32) {
 		info_free_clusters
@@ -644,7 +646,7 @@ _get_min_resize_size (const PedFileSystem* fs, PedSector min_data_size)
 	}
 
 /* adds a bit of leeway (64 sectors), for resolving extra issues, like root
- * directory allocation, that aren't covered here. 
+ * directory allocation, that aren't covered here.
  */
 	return max_length + 64;
 }
