@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2004, 2005, 2007 Free Software Foundation, Inc.
+    Copyright (C) 2004-2005, 2007, 2009 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ hfs_extent_key_cmp(HfsPrivateGenericKey* a, HfsPrivateGenericKey* b)
 	/* 0xFFFFFFFF > 1 !!! (this is the 2.4 bug) */
 	if (key1->file_ID != key2->file_ID)
 		return PED_BE32_TO_CPU(key1->file_ID) <
-		       PED_BE32_TO_CPU(key2->file_ID) ? 
+		       PED_BE32_TO_CPU(key2->file_ID) ?
 				-1 : +1;
 
 	if (key1->type != key2->type)
@@ -64,7 +64,7 @@ hfs_extent_key_cmp(HfsPrivateGenericKey* a, HfsPrivateGenericKey* b)
 		return 0;
 	/* the whole thing wont work with 16 bits ints */
 	/* anyway */
-	return (int)( PED_BE16_TO_CPU(key1->start) - 
+	return (int)( PED_BE16_TO_CPU(key1->start) -
 		      PED_BE16_TO_CPU(key2->start) );
 }
 
@@ -133,7 +133,7 @@ hfs_btree_search (HfsPrivateFile* b_tree_file, HfsPrivateGenericKey* key,
 			if (!hfs_file_read_sector(b_tree_file, node,
 						  node_number))
 				return 0;
-		} else 
+		} else
 			break;
 	}
 
@@ -161,7 +161,7 @@ hfs_free_bad_blocks_list(HfsPrivateLinkExtent* first)
 
 	while (first) {
 		next = first->next;
-		ped_free (first);
+		free (first);
 		first = next;
 	}
 }
@@ -178,7 +178,7 @@ hfs_read_bad_blocks (const PedFileSystem *fs)
 		return 1;
 
 	{
-	uint8_t			record[sizeof (HfsExtentKey) 
+	uint8_t			record[sizeof (HfsExtentKey)
 					       + sizeof (HfsExtDataRec)];
 	HfsExtentKey		search;
 	HfsExtentKey*		ret_key = (HfsExtentKey*) record;
@@ -248,7 +248,7 @@ hfs_is_bad_block (const PedFileSystem *fs, unsigned int fblock)
 	for (walk = priv_data->bad_blocks_xtent_list; walk; walk = walk->next) {
 		/* Won't compile without the strange cast ! gcc bug ? */
 		/* or maybe C subtilties... */
-		if ((fblock >= PED_BE16_TO_CPU (walk->extent.start_block)) && 
+		if ((fblock >= PED_BE16_TO_CPU (walk->extent.start_block)) &&
 		    (fblock <  (unsigned int) (PED_BE16_TO_CPU (
 						    walk->extent.start_block)
 					       + PED_BE16_TO_CPU (
@@ -268,19 +268,19 @@ hfs_get_empty_end (const PedFileSystem *fs)
 	HfsPrivateFSData* 	priv_data = (HfsPrivateFSData*)
 						fs->type_specific;
 	HfsMasterDirectoryBlock* mdb = priv_data->mdb;
-	HfsPrivateLinkExtent*	link;
 	unsigned int		block, last_bad, end_free_blocks;
 
 	/* find the next block to the last bad block of the volume */
 	if (!hfs_read_bad_blocks (fs))
 		return 0;
 
+	HfsPrivateLinkExtent*	l;
 	last_bad = 0;
-	for (link = priv_data->bad_blocks_xtent_list; link; link = link->next) {
-		if ((unsigned int) PED_BE16_TO_CPU (link->extent.start_block)
-		    + PED_BE16_TO_CPU (link->extent.block_count) > last_bad)
-			last_bad = PED_BE16_TO_CPU (link->extent.start_block)
-			           + PED_BE16_TO_CPU (link->extent.block_count);
+	for (l = priv_data->bad_blocks_xtent_list; l; l = l->next) {
+		if ((unsigned int) PED_BE16_TO_CPU (l->extent.start_block)
+		    + PED_BE16_TO_CPU (l->extent.block_count) > last_bad)
+			last_bad = PED_BE16_TO_CPU (l->extent.start_block)
+			           + PED_BE16_TO_CPU (l->extent.block_count);
 	}
 
 	/* Count the free blocks from last_bad to the end of the volume */
